﻿-------------------------------------------------------------------------------
-- Local data
-------------------------------------------------------------------------------
local addonName, addon = ...
local addonNameDB = addonName.."DB"
local L = addon.L
local PET_TRACKER = addon.PET_TRACKER
local wowWoD = (select(4, GetBuildInfo())) >= 60000


-------------------------------------------------------------------------------
-- Database
-------------------------------------------------------------------------------
local db
local spells
if wowWoD then
	spells = {
		(GetSpellInfo(30455)),
		(GetSpellInfo(8921)),
	}
else
	spells = {
		(GetSpellInfo(122475)),
		(GetSpellInfo(30455)),
		(GetSpellInfo(8921)),
		(GetSpellInfo(77799)),
		(GetSpellInfo(1978)),
	}
end
local defaults = {
	version = 1,
	anchor = {
		point = "CENTER",
		relativePoint = "CENTER",
		xOffset = 0,
		yOffset = 0,
	},
	options = {
		hideButton = false,
		maxDistance = 80,
		usePetTracker = false,
		findMacroText = "toll,elox:24,robo",
		killMacroText = table.concat(spells, ","),
	},
}


-------------------------------------------------------------------------------
-- Tutorial
-------------------------------------------------------------------------------
local helpPlate = {}

local function ToggleTutorial(button, parent)
	if not HelpPlate_IsShowing(helpPlate) then
		local self = helpPlate
		if not self.FramePos then
			self.FramePos = {}
		end
		if not self.FrameSize then
			self.FrameSize = {}
		end
		local xOffset = 0
		local yOffset = -button:GetHeight()
		self.FramePos.x = xOffset
		self.FramePos.y = yOffset
		self.FrameSize.width  = xOffset + parent:GetWidth()
		self.FrameSize.height = yOffset + parent:GetHeight()
		for k,v in ipairs(self) do
			if v.Frame then
				if not v.HighLightBox then
					v.HighLightBox = {}
				end
				v.HighLightBox.x = v.Frame:GetLeft() - parent:GetLeft() - xOffset
				v.HighLightBox.y = v.Frame:GetTop()  - parent:GetTop()  - yOffset
				v.HighLightBox.width  = v.Frame:GetWidth()
				v.HighLightBox.height = v.Frame:GetHeight()
				if not v.ButtonPos then
					v.ButtonPos = {}
				end
				v.ButtonPos.x = v.HighLightBox.x - 23 + v.HighLightBox.width / 2
				v.ButtonPos.y = v.HighLightBox.y + 23 - v.HighLightBox.height / 2
				if not v.ToolTipDir then
					v.ToolTipDir = "RIGHT"
				end
			end
		end
		HelpPlate_Show(self, parent, button, true)
	else
		HelpPlate_Hide(true)
	end
end

local function HideTutorial()
	if HelpPlate_IsShowing(helpPlate) then
		HelpPlate_Hide(true)
	end
end

local function SetTutorial(parent, relativeTo)
	local button = CreateFrame("Button", "$parentTutorialButton", parent, "MainHelpPlateButton")
	button:SetPoint("LEFT", relativeTo, "RIGHT")
	button:SetScript("OnClick", function() ToggleTutorial(button, parent) end)
	parent:SetScript("OnHide", HideTutorial)
end


-------------------------------------------------------------------------------
-- Local functions
-------------------------------------------------------------------------------
local function CreatePanel(name, title, parent)
	local frame = CreateFrame("Frame", addonName..name)
	frame.name = title
	frame.parent = parent
	return frame
end

local function SetText(self, text, relativeTo, height)
	if relativeTo then
		self:SetPoint("TOPLEFT", relativeTo, "BOTTOMLEFT", 0, -8)
	else
		self:SetPoint("TOPLEFT", 16, -16)
	end
	if height then
		self:SetHeight(height)
	end
	if text and text:len() > 50 then
		self:SetPoint("RIGHT", -16, 0)
	end
	self:SetJustifyV("TOP")
	self:SetText(text)
	return self
end

local function HideButton(setting)
	if not InCombatLockdown() then
		if setting == "1" or setting == true then
			addon.clickButton:Hide()
		else
			addon.clickButton:Show()
		end
	end
end


-------------------------------------------------------------------------------
-- Initializes the database.
-------------------------------------------------------------------------------
function addon:InitDatabase()
	db = _G[addonNameDB]
	if not db then
		db = CopyTable(defaults)
		_G[addonNameDB] = db
	end
	db.usePetTracker = _G[PET_TRACKER] and db.usePetTracker or false
	return db
end


-------------------------------------------------------------------------------
-- Creates the interface panels.
-------------------------------------------------------------------------------
function addon:CreateInterfacePanels()
	self:CreateOptionsPanel()
	self:CreateHelpPanel()
end


-------------------------------------------------------------------------------
-- Creates the options panel.
-------------------------------------------------------------------------------
function addon:CreateOptionsPanel()
	local panel = CreatePanel("OptionsPanel", self.title)
	panel.options = CopyTable(db.options)

	panel.okay = function(self)
		--print(self:GetName(), "okay")
		db.options.hideButton = self.HideButton:GetChecked()
		db.options.maxDistance = self.MaxDistance:GetValue()
		db.options.usePetTracker = self.UsePetTracker:GetChecked()
		db.options.findMacroText = self.FindEditBox:GetText()
		db.options.killMacroText = self.KillEditBox:GetText()
		self.options = CopyTable(db.options)
		addon:UpdateFindMacro()
		addon:UpdateKillMacro()
	end

	panel.cancel = function(self)
		--print(self:GetName(), "cancel")
		self.options = CopyTable(db.options)
		HideButton(self.options.hideButton)
	end

	panel.default = function(self)
		--print(self:GetName(), "default")
		self.options = CopyTable(defaults.options)
		HideButton(self.options.hideButton)
	end

	panel.refresh = function(self)
		--print(self:GetName(), "refresh")
		self.HideButton:SetChecked(self.options.hideButton)
		self.MaxDistance:SetValue(self.options.maxDistance)
		self.UsePetTracker:SetChecked(self.options.usePetTracker)
		self.FindEditBox:SetEnabled(not self.options.usePetTracker)
		self.FindEditBox:SetAlpha((self.options.usePetTracker and 1 or 2) * 0.5)
		self.FindEditBox:SetText(self.options.findMacroText)
		self.KillEditBox:SetText(self.options.killMacroText)
	end

	local function CreateEditFrame(name, parent, relativeTo, height)
		local frame = CreateFrame("Frame", nil, parent)
		frame:SetPoint("TOPLEFT", relativeTo, "BOTTOMLEFT", 0, -8)
		frame:SetPoint("RIGHT", -16, 0)
		frame:SetHeight(height)
		frame:SetBackdrop({
			bgFile = [[Interface\Tooltips\UI-Tooltip-Background]],
			edgeFile = [[Interface\Tooltips\UI-Tooltip-Border]],
			tile = true, edgeSize = 16, tileSize = 16,
			insets = { left = 5, right = 5, top = 5, bottom = 5 }
		});
		local color = TOOLTIP_DEFAULT_BACKGROUND_COLOR
		frame:SetBackdropColor(color.r, color.g, color.b)
		color = TOOLTIP_DEFAULT_COLOR
		frame:SetBackdropBorderColor(color.r, color.g, color.b)

		local editBox = CreateFrame("EditBox")
		local scrollFrame = CreateFrame("ScrollFrame", parent:GetName()..name, frame, "UIPanelScrollFrameTemplate")
		scrollFrame:SetPoint("TOPLEFT", 10, -6)
		scrollFrame:SetPoint("BOTTOMRIGHT", -28, 5)
		scrollFrame:SetScrollChild(editBox)
		scrollFrame:SetScript("OnSizeChanged", function(self, width, height) editBox:SetWidth(width - 6) end)

		editBox:SetAutoFocus(false)
		editBox:SetMultiLine(true)
		editBox:SetFontObject("ChatFontNormal")
		editBox:SetScript("OnEscapePressed", EditBox_ClearFocus)
		editBox:SetScript("OnCursorChanged", ScrollingEdit_OnCursorChanged)
		--editBox:SetScript("OnTextChanged", function(self, userInput) ScrollingEdit_OnTextChanged(self, scrollFrame) end )
		editBox:SetScript("OnUpdate", ScrollingEdit_OnUpdate)

		local button = CreateFrame("Button", nil, panel)
		button:SetAllPoints(scrollFrame)
		button:SetScript("OnClick", function() editBox:SetFocus() end)

		frame.EditBox = editBox
		return frame
	end

	local item
	item = SetText(panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLargeLeft"), self.title)
	SetTutorial(panel, item)
	item = SetText(panel:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft"), L.OPTIONS_INFO, item, 32)

	local check1 = CreateFrame("CheckButton", nil, panel, "InterfaceOptionsCheckButtonTemplate")
	check1:SetPoint("TOPLEFT", item, "BOTTOMLEFT")
	check1.Text:SetText(L["OPTIONS_HIDE_BUTTON"])
	check1.tooltipText = L["OPTIONS_HIDE_BUTTON_TOOLTIP"]
	check1.setFunc = HideButton
	panel.HideButton = check1

	local slider1 = CreateFrame("Slider", "$parentDistance", check1, "OptionsSliderTemplate")
	slider1:SetWidth(200)
	slider1:SetPoint("TOPLEFT", item, "BOTTOMLEFT", 10, -70)
	slider1:SetScript("OnValueChanged", function(self, value, smooth)
		local min = self:GetMinMaxValues()
		local step = self:GetValueStep()
		local val = min + ceil((value - min) / step) * step
		if val ~= value then
			self:SetValue(val)
		else
			_G[self:GetName().."Text"]:SetText(format(L.OPTIONS_MAX_DISTANCE, value))
		end
	end)
	slider1:SetMinMaxValues(0, 80)
	slider1:SetValueStep(5)
	slider1:SetValue(5)
	_G[slider1:GetName().."Low"]:SetText(0)
	_G[slider1:GetName().."High"]:SetText(80)
	slider1.tooltipText = L.OPTIONS_MAX_DISTANCE_TOOLTIP
	panel.MaxDistance = slider1

	local check2 = CreateFrame("CheckButton", nil, panel, "InterfaceOptionsCheckButtonTemplate")
	check2:SetPoint("TOPLEFT", slider1, "BOTTOMLEFT", -10, -24)
	check2.Text:SetText(format(L.OPTIONS_PET_TRACKER, PET_TRACKER))
	check2.setFunc = function(setting)
		if not _G[PET_TRACKER] then
			check2:SetChecked(false)
			return
		end
		setting = setting == "0"
		panel.FindEditBox:SetEnabled(setting)
		panel.FindEditBox:SetAlpha(setting and 1 or 0.5)
	end
	if _G[PET_TRACKER] then
		check2.tooltipText = format(L.OPTIONS_PET_TRACKER_TOOLTIP1, PET_TRACKER)
	else
		check2.tooltipText = format(L.OPTIONS_PET_TRACKER_TOOLTIP2, PET_TRACKER)
		check2.tooltipRequirement = format(L.OPTIONS_PET_TRACKER_TOOLTIP3, PET_TRACKER)
	end
	panel.UsePetTracker = check2

	item = SetText(panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft"), L.OPTIONS_FIND_LIST, check2)
	item = CreateEditFrame("Find", panel, item, 90)
	panel.FindEditBox = item.EditBox
	table.insert(helpPlate, {Frame = item, ToolTipText = L.TUTORIAL1})

	item = SetText(panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft"), L.OPTIONS_KILL_LIST, item)
	item = CreateEditFrame("Kill", panel, item, 90)
	panel.KillEditBox = item.EditBox
	table.insert(helpPlate, {Frame = item, ToolTipText = L.TUTORIAL2})

	InterfaceOptions_AddCategory(panel)
end


-------------------------------------------------------------------------------
-- Creates the help panel.
-------------------------------------------------------------------------------
function addon:CreateHelpPanel()
	local panel = CreatePanel("HelpPanel", HELP_LABEL, self.title)

	local child = CreateFrame("Frame")
	child:SetSize(1,1)

	local item
	item = SetText(panel:CreateFontString(nil, "ARTWORK", "GameFontNormalLargeLeft"), self.title.." - "..HELP_LABEL)
	item = SetText(panel:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft"), L.HELP1, item, 48)
	item = SetText(child:CreateFontString(nil, "ARTWORK", "GameFontNormalLeft"), L.HELP2)
	item = SetText(child:CreateFontString(nil, "ARTWORK", "GameFontHighlightLeft"), L.HELP3, item)

	local frame = CreateFrame("ScrollFrame", "$parentScrollFrame", panel, "UIPanelScrollFrameTemplate")
	frame:SetPoint("TOPLEFT", 0, -100)
	frame:SetPoint("BOTTOMRIGHT", -28, 16)
	frame:SetScrollChild(child)
	frame:SetScript("OnSizeChanged", function(self, width) self:GetScrollChild():SetWidth(width) end)

	InterfaceOptions_AddCategory(panel)
end

